use strict;
use warnings;
##use Data::Dumper;

#date modified 4/1
# This program takes GO/GONE annotations and produces a list of annotation in GAF2.0 format to be used with our GSEA program to make gene sets. The program extracts human annotation (identified by comparing UniProts to list of human UniProt IDs downloaded from their website) and also transfers mouse annotation to human orthologs (determined by comparing UniProt IDs to an ortholog list given to us by the Jax)
# For part of the GAF2.0 format, the program then looks up the ontology of each annotation using a terms/IDs file downloaded from the GO and added to manually if missing terms are found. Finally, the program uses a file containing a uniprot-to-official-gene-symbol file and inserts the official gene symbol into each annotation (and copies the annotation if there is more than one gene symbol for a given uniprot ID).
# The program produces 4 output files:
# MGI_UniProt_Orthologs.txt -- mouse MGI and their human UniProt orthologs
# MissingPFC.txt -- GO IDs that are missing from the terms/ID file that has their ontology
# MissingGS.txt -- UniProt IDs that don't have Gene Symbols (these annotations still appear in OrthGAF2.0.txt)
# OrthGAF2.0.txt -- The human developmental annotation, in GAF 2.0 format (except the last 2 columns which instead contain additional info: the original mouse evidence code and the text part of the GO term)


my $UniProt; #Human UniProt ID ortholog from Human_SP_IDs.txt (Value of hash %MGItoUP)
my $MGI; #Mouse MGI ID from Human_SP_Ids.txt (key of hash %MGItoUP)
my %MGItoUP; #Hash of mouse MGI keys and their human UniProt ID values

# Command line takes in mouse annotation file
if ($#ARGV != 0) {
   die "Usage: GAF2.0.GONE.pl <filename.txt> \n";
}

# Open the file that contains HGNC, UniProt, and MGI IDs in that order, tab-delimited.
# This file was obtained from MGI
my $genefile = "Input_Mappings/Human_SP_IDs.txt"; #extra placement tabs for lines with no MGI removed
unless (open(FILE, $genefile)) {
    die "Couldn't open $genefile\n";
}

# Each line is split by \t into 3 elements of an array, @IDs. Skip the line if the 2nd and 3rd element are undefined (ie there is no UniProt or MGI ID). Otherwise, the 3rd element is the MGI ID ($MGI) 2nd element is the UniProt ID ($UniProt) which become the keys and values, respectively, of the hash %HumanID.
my $header = <FILE>;
my @lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
	my @current_line = split /\t/, $line;
      	next if (!($current_line[1])  or !($current_line[2]));
#      	next if ((!($current_line[1]) or ($current_line[1]) eq "") or (!($current_line[2]) or ($current_line[2]) eq ""));
	$MGI = $current_line[2];
	$UniProt = $current_line[1];
	$MGItoUP{$MGI} = $UniProt;
	}
close(FILE);

##print Dumper(keys %MGItoUP);

# Write the hash of MGI and human UniProt ortholog IDs to a file
unless (open(FILE, ">Output/MGI_UniProt_Orthologs.DFLAT.txt")) {
    die "Couldn't open MGI_UniProt_Orthologs to write!\n"
}
foreach $MGI (sort keys %MGItoUP) {
    print FILE "$MGI\t$MGItoUP{$MGI}\n";
}
close(FILE);


my $mUniProt; #Mouse UniProt ID from MGImouseUP.txt (key of hash %MGItoUP)
my $mMGI; #MGI ID from MGImouseUP.txt (value of hash %MGItoUP)
my %MUP; #Hash of mouse UniProt keys and their MGI values

# Open the file that contains MGI and UniProt IDs in that order, tab-delimited.
# This file was obtained from the UniProt website
my $mousefile = "Input_Mappings/MGImouseUP.txt";
unless (open(FILE, $mousefile)) {
    die "Couldn't open $mousefile\n";
}

# Read the header and lines. Split lines to extract MGI and UniProt IDs. Set UniProts as keys and MGIs as values. Note that $mMGI contains several MGI IDs (will be split later)
$header = <FILE>;
@lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
	my @current_line = split /\t/, $line;
	$mUniProt = $current_line[1];
	$mMGI = $current_line[0];
	next if (!$mUniProt or !$mMGI);
	if (exists $MUP{$mUniProt}) {
		$MUP{$mUniProt} = join "\t", $MUP{$mUniProt}, $mMGI;
	} else {
		$MUP{$mUniProt} = $mMGI;
	}
}
close(FILE);

# Write the hash of MGI and UniProt IDs to a file
unless (open(FILE, ">Output/mouse_MGI_UP.DFLAT.txt")) {
    die "Couldn't open MGI_UniProt_Orthologs to write!\n"
}
foreach $mUniProt (sort keys %MUP) {
    print FILE "$mUniProt\t$MUP{$mUniProt}\n";
}
close(FILE);


my $GO; #GO ID number; key for %ontology hash
my $PFC; #Ontology (P, F, or C); value for %ontology hash
my %Ontology; #Hash of GO IDs and their Ontologies
my @MissingPFC; #List of GO IDs that don't exist in the GO terms/ontology file

# Open the file that contains the P, F, or C ontology for each GO term
# This file was downloaded from the GO
my $pfcfile = "Input_Mappings/GO.terms_alt_ids.txt"; #GO terms/ontology file
unless (open(FILE, $pfcfile)) {
	die "Couldn't open $pfcfile!\n";
}

# Read each line. Skip the header lines (start with !). Split each line and make a hash of the GO IDs as keys and their P, F, or C ontology as the values
@lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
      	next if ($line =~ /^!/);
      	$line =~ tr/\n\r//d; #removes carriage returns
	my @current_line = split /\t/, $line;
	$GO = $current_line[0];
	$PFC = $current_line[3];
	$Ontology{$GO} = $PFC;
	}
close(FILE);

my %GeneHash; #A hash with human UniProt IDs as keys and an array of their Gene Symbols as keys
my $UP; #Human UniProt keys in the %GeneHash
my $GS; #Human GS values in the %GeneHash
my $current_UP; #current UniProt ID
my $current_annotation; #Annotation line being created in GAF2.0 format, derived from annotation file, for a given MGI/UniProt ortholog
my %AnnotationHash; #Hash of MGI/UniProt keys ($two_IDs) and annotation values (@annotations)
my $two_IDs; #MGI/UniProt key of %AnnotationHash
my %MissingAnnotationHash; #Hash of annotations with missing GS (UniProt does not exist in %GeneHash
my @missingGS; #Array of UniProt IDs that don't have Gene Symbols

my @MGIs; #Array of MGIs created by splitting the $mMGI values in the %MUP
my $current_MGI;


# Open the file that has the uniprot-genesymbol conversion (two columns, tab-delimited)
# This file has to be made by the user by submitting UniProt IDs to DAVID
my $GSfile = "Input_Mappings/uniprot-9606.tab";
unless (open(FILE, $GSfile)) {
	die "Couldn't open $GSfile\n";
}

# Split each line and create a hash with the UniProt ($UP) as the keys and the first gene symbol ($GS) as the values. Skip 
$header = <FILE>;
@lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
      	$line =~ tr/\n\r//d; #removes carriage returns	
	($UP, $GS) = split /\s/, $line;
	next if (!$GS);
	$GeneHash{$UP} = $GS;
}
close(FILE);

# Print %GeneHash to a file
unless (open(FILE, ">Output/UPtoGS.DFLAT.txt")) {
	die "Couldn't open Orth_GAF2.0 to write!\n";
}
foreach $UP (sort keys %GeneHash) {
	print FILE "$UP\t$GeneHash{$UP}\n";
}

# Open the file that has the annotations you want to search through
# Format:

unless (open (IN1, $ARGV[0])) {
   die "cannot open $ARGV[0] for input\n";
}

##my $annotationfile = "P2GO4.10.13.txt";
##unless (open(FILE, $annotationfile)) {
##	die "Couldn't open $annotationfile\n";
##}

##@lines = <FILE>;
##close(FILE);
while (<IN1>) {
	@lines = <IN1>;
	foreach my $line (@lines) {
		chomp $line;
		$line =~ tr/\n\r//d; #removes carriage returns	
		my @current_line = split /\t/, $line;
# Skip if the annotation was deleted
		next if ($current_line[11] eq "D");
#		$current_UP = $current_line[2];
# Check to make sure that every GO ID exists in the Ontology hash with an ontology designation of P, F, or C. If not, create the GO ID in the Ontology hash and give it an ontology of X for easy lookup/correction later. @MissingPFC are GO IDs with no ontology in the file
		if (!exists $Ontology{$current_line[3]}) {
			$Ontology{$current_line[3]} = "X";
			push (@MissingPFC, $current_line[3])
		}
# If the UniProt ID is human (exists in %GeneHash), make a GAF2.0 annotation based on it in the %AnnotationHash
		if (exists $GeneHash{$current_line[2]}) {
			$current_UP = $current_line[2];
			$two_IDs = "$current_line[2] (human)";
			$current_annotation = join "\t", "UniProtKB", $current_line[2], $GeneHash{$current_UP}, $current_line[9], $current_line[3], "DFLAT from P2GO", $current_line[4], $current_line[2], $Ontology{$current_line[3]}, "", "", "", "taxon:9606", "20110126", "DFLAT", "original: $current_line[4]", $current_line[3];
# If the @two_IDs key already has an annotation value, join the $current_annotation to it
			if (exists $AnnotationHash{$two_IDs}) {
				$AnnotationHash{$two_IDs} = join "\n", $AnnotationHash{$two_IDs}, $current_annotation;
				} else {
					$AnnotationHash{$two_IDs} = $current_annotation;
				}
# If the UniProt isn't in %GeneHash already, it's not human. See if it's a mouse ortholog
		} else {
# First, skip it if it has an unacceptable evidence code (ISS, ISO, ISA, ISM, RCA, IGC, IEA, TAS, NAS, IC, ND, or NR).
			next if (($current_line[4] =~ /IS/) or ($current_line[4] =~ /RCA/) or ($current_line[4] =~ /IGC/) or ($current_line[4] =~ /IEA/) or ($current_line[4] =~ /AS/)or ($current_line[4] =~ /IC/) or ($current_line[4] =~ /ND/) or ($current_line[4] =~ /NR/));
# If the UniProt ($current_line[2]) exists in the mouse UP/MGI hash %MUP from above, then we can use the ortholog. Set $last_UP to the current UP
			if (exists $MUP{$current_line[2]}) {
				@MGIs = split /\t/, $MUP{$current_line[2]};
				foreach (@MGIs) {
					$current_MGI = $_;
# @two_IDs and $two_IDs are the MGI ID and mouse UniProt Ortholog; this is a key for the %AnnotationHash
					if (exists $MGItoUP{$current_MGI}) {
						$current_UP = $MGItoUP{$current_MGI};
						my @two_IDs = ($current_MGI, "$current_line[2] (mouse)");
						$two_IDs = join "\n", @two_IDs;
# If the human UniProt ortholog ($current_UP) exists in %GeneHash, save the annotation information in modified GAF 2.0 format: UniProtKB, UniProt, Gene Symbol, blank, GO term ID, MGI Ortholog list, Evidence Code (ISS), original UniProt ID, Ontology (F, P, or C; X if missing), Gene description, blank, blank, Taxon, Date, DFLAT, Original Evidence Code, GO term
						if (exists $GeneHash{$current_UP}) {
							$current_annotation = join "\t", "UniProtKB", $MGItoUP{$current_MGI}, $GeneHash{$current_UP}, $current_line[9], $current_line[3], "DFLAT with MGI Ortholog list", "ISS", $current_line[2], $Ontology{$current_line[3]}, "", "", "", "taxon:9606", "20110126", "DFLAT", "original: $current_line[4]", $current_line[3];
							if (exists $AnnotationHash{$two_IDs}) {
								$AnnotationHash{$two_IDs} = join "\n", $AnnotationHash{$two_IDs}, $current_annotation;
								} else {
									$AnnotationHash{$two_IDs} = $current_annotation;
								}
# If the $current_UP isn't in %GeneHash, no gene symbol exists, so save the annotations to a separate %MissingAnnotationHash
						} else {
							$current_annotation = join "\t", "UniProtKB", $MGItoUP{$current_MGI}, "", $current_line[9], $current_line[3], "DFLAT with MGI Ortholog list", "ISS", $current_line[2], $Ontology{$current_line[3]}, "", "", "", "taxon:9606", "20110126", "DFLAT", "original: $current_line[4]", $current_line[3];
							if (exists $AnnotationHash{$two_IDs}) {
								$AnnotationHash{$two_IDs} = join "\n", $AnnotationHash{$two_IDs}, $current_annotation;
								} else {
									$AnnotationHash{$two_IDs} = $current_annotation;
								}
#				push (@missingGS, $two_IDs);			#simple array of missing IDs if you want to print these later
						}
###		$AnnotationHash{$two_IDs} = join "\n", @annotations; 	#uncomment if you want all annotation together regardless of missing IDs
					}
				}
			}
		}
	}
}

# Write file of GO terms that don't have ontology (P, F, or C) 
open(FILE, ">Output/MissingPFC.DFLAT.txt");
foreach (@MissingPFC){
print FILE "$_\n";
}

# Write file of UniProt IDs that are missing Gene Symbols
open(FILE, ">Output/MissingGS.DFLAT.txt");
foreach $two_IDs (sort keys %MissingAnnotationHash){
	print FILE "$MissingAnnotationHash{$two_IDs}\n";
}

# Print the %annotation hash for the user
# Printed in GAF 2.0 format, except that the last 2 columns are the former ID and the go term text
##foreach $UPI (sort keys %Annotation) {
##	print "$Annotation{$UPI}\n\n";
##}
unless (open(FILE, ">Output/Orth_GAF2.0.DFLAT.txt")) {
	die "Couldn't open Orth_GAF2.0 to write!\n";
}
foreach $two_IDs (sort keys %AnnotationHash) {
	print FILE "$AnnotationHash{$two_IDs}\n";
}
close(FILE);
