use strict;
use warnings;

# This program takes GO/GONE annotations and produces a list of annotation in GAF2.0 format to be used with our GSEA program to make gene sets. The program extracts human annotation (identified by comparing UniProts to list of human UniProt IDs downloaded from their website) and also transfers mouse annotation to human orthologs (determined by comparing UniProt IDs to an ortholog list given to us by the Jax)
# For part of the GAF2.0 format, the program then looks up the ontology of each annotation using a terms/IDs file downloaded from the GO and added to manually if missing terms are found. Finally, the program uses a file containing a uniprot-to-official-gene-symbol file and inserts the official gene symbol into each annotation (and copies the annotation if there is more than one gene symbol for a given uniprot ID).
# The program produces 4 output files:
# MGI_UniProt_Orthologs.txt -- mouse MGI and their human UniProt orthologs
# MissingPFC.txt -- GO IDs that are missing from the terms/ID file that has their ontology
# MissingGS.txt -- UniProt IDs that don't have Gene Symbols (these annotations still appear in OrthGAF2.0.txt)
# OrthGAF2.0.txt -- The human developmental annotation, in GAF 2.0 format (except the last 2 columns which instead contain additional info: the original mouse evidence code and the text part of the GO term)


my $UniProt; #Human UniProt ID ortholog from Human_SP_Ids.txt (Value of hash %MGItoUP)
my $MGI; #Mouse MGI ID from Human_SP_Ids.txt (key of hash %MGItoUP)
my %MGItoUP; #Hash of mouse MGI keys and their human UniProt ID values

# Command line takes in mouse annotation file
if ($#ARGV != 0) {
   die "Usage: GAF2.0.GONE.pl <filename.txt> \n";
}

# Open the file that contains HGNC, UniProt, and MGI IDs in that order, tab-delimited.
# This file was obtained from MGI
my $genefile = "Input_Mappings/Human_SP_IDs.txt"; #extra placement tabs for lines with no MGI removed
unless (open(FILE, $genefile)) {
    die "Couldn't open $genefile\n";
}

# Read in the header of $genefile
my $header = <FILE>;

# Each line is split by \t into 3 elements of an array, @IDs. Skip the line if the 2nd and 3rd element are undefined (ie there is no UniProt or MGI ID). Otherwise, the 3rd element is the MGI ID ($MGI) 2nd element is the UniProt ID ($UniProt) which become the keys and values, respectively, of the hash %HumanID.
my @lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
	my @current_line = split /\t/, $line;
      	next if (!($current_line[1]) or !($current_line[2]));
	$MGI = $current_line[2];
	$UniProt = $current_line[1];
	$MGItoUP{$MGI} = $UniProt;
	}
close(FILE);

# Print the hash of MGI/UniProt IDs for the user:
##foreach $MGI (sort keys %MGItoUP) {
##	print "$MGI\t$MGItoUP{$MGI}\n";
##}

# Write the hash of MGI and UniProt IDs to a file
unless (open(FILE, ">Output/MGI_UniProt_Orthologs.GONE.txt")) {
    die "Couldn't open MGI_UniProt_Orthologs to write!\n"
}
foreach $MGI (sort keys %MGItoUP) {
    print FILE "$MGI\t$MGItoUP{$MGI}\n";
}
close(FILE);

#-------------------------
my $mUniProt; #Mouse UniProt ID from MGImouseUP.txt (Value of hash %MGItoUP)
my $mMGI; #MGI ID from MGImouseUP.txt (key of hash %MGItoUP)
my %MUP; #Hash of Mouse keys and their mouse UniProt ID values

# Open the file that contains HGNC, UniProt, and MGI IDs in that order, tab-delimited.
# This file was obtained from MGI
my $mousefile = "Input_Mappings/MGImouseUP.txt";
unless (open(FILE, $mousefile)) {
    die "Couldn't open $mousefile\n";
}

# Each line is split by \t into 3 elements of an array, @IDs. Skip the line if the 2nd and 3rd element are undefined (ie there is no UniProt or MGI ID). Otherwise, the 3rd element is the MGI ID ($MGI) 2nd element is the UniProt ID ($UniProt) which become the keys and values, respectively, of the hash %HumanID.
$header = <FILE>;
@lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
	my @current_line = split /\t/, $line;
	$mUniProt = $current_line[1];
	$mMGI = $current_line[0];
	next if (!$mUniProt or !$mMGI);
	if (exists $MUP{$mUniProt}) {
		$MUP{$mUniProt} = join "\t", $MUP{$mUniProt}, $mMGI;
	} else {
		$MUP{$mUniProt} = $mMGI;
	}
}
close(FILE);

# Print the hash of mouse UniProt/MGI IDs for the user:
##foreach $mUniProt (sort keys %MUP) {
##	print "$mUniProt\t$MUP{$mUniProt}\n";
##}

# Write the hash of MGI and UniProt IDs to a file
unless (open(FILE, ">Output/mouse_MGI_UP.GONE.txt")) {
    die "Couldn't open MGI_UniProt_Orthologs to write!\n"
}
foreach $mUniProt (sort keys %MUP) {
    print FILE "$mUniProt\t$MUP{$mUniProt}\n";
}
close(FILE);

#-------------------------
my $GO; #GO ID number; key for %ontology hash
my $PFC; #Ontology (P, F, or C); value for %ontology hash
my %Ontology; #Hash of GO IDs and their Ontologies
my @MissingPFC; #List of GO IDs that don't exist in the GO terms/ontology file

# Open the file that contains the P, F, or C ontology for each GO term
# This file was downloaded from the GO
my $pfcfile = "Input_Mappings/GO.terms_alt_ids.txt"; #GO terms/ontology file
unless (open(FILE, $pfcfile)) {
	die "Couldn't open $pfcfile!\n";
}

# Read each line. Skip the header lines (start with !). Split each line and make a hash of the GO IDs as keys and their P, F, or C ontology as the values
@lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
      	next if ($line =~ /^!/);
	my @current_line = split /\t/, $line;
	$GO = $current_line[0];
	$PFC = $current_line[3];
	$Ontology{$GO} = $PFC;
	}
close(FILE);

# Print the Ontology hash for the user
##foreach $GO (keys %Ontology) {
##print "$GO\t$Ontology{$GO}\n";
##}

my %GeneHash; #A hash with UniProt IDs as keys and an array of gene symbols as keys
my $UP; #Human UniProt keys in the %GeneHash
my $GS; #Human GS values in the %GeneHash
my $current_UP; #current UniProt ID
my $current_annotation; #Annotation line being created in GAF2.0 format, derived from annotation file, for a given MGI/UniProt ortholog
my @annotations = (); #array of annotations ($current_annotation)s for each MGI/UniProt
my %AnnotationHash; #Hash of MGI/UniProt keys ($two_IDs) and annotation values (@annotations)
my $two_IDs; #MGI/UniProt key of %AnnotationHash
my $last_UP = ""; #The previous UP; compared to current MGI ($currentPline[0] to reset @annotations if working with new MGI
my %MissingAnnotationHash; #Hash of annotations with missing GS (UniProt does not exist in %GeneHash
my @missingGS; #Array of UniProt IDs that don't have Gene Symbols

my @MGIs;
my $current_MGI;
my $last_MGI;

# Open the file that has the uniprot-genesymbol conversion (two columns, tab-delimited)
# This file has to be made by the user by submitting UniProt IDs to DAVID
my $GSfile = "Input_Mappings/uniprot-9606.tab";
unless (open(FILE, $GSfile)) {
	die "Couldn't open $GSfile\n";
}

# Split each line and create a hash with the UniProt ($UP) as the keys and the gene symbols ($GS) as the values. If the UniProt already exists in the hash, add the additional gene symbol to the value separated by a tab
$header = <FILE>;
@lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
	($UP, $GS) = split /\s/, $line;
	next if (!$GS);
	$GeneHash{$UP} = $GS;
}
close(FILE);

# Print %GeneHash for the user
##foreach $UP (sort keys %GeneHash) {
##	print "$UP\t$GeneHash{$UP}\n";
##}

# Print %GeneHash to a file
unless (open(FILE, ">Output/UPtoGS.GONE.txt")) {
	die "Couldn't open Orth_GAF2.0 to write!\n";
}
foreach $UP (sort keys %GeneHash) {
	print FILE "$UP\t$GeneHash{$UP}\n";
}

# Open the file that has the annotations you want to search through
# This file was obtained from MGI
# Please note that the annotation file has to be sorted by UniProt ID (all annotations for a given ID must be blocked together)

unless (open (IN1, $ARGV[0])) {
   die "cannot open $ARGV[0] for input\n";
}

##my $annotationfile = "GONE4.10.13.txt";
##unless (open(FILE, $annotationfile)) {
##	die "Couldn't open $annotationfile\n";
##}
##$header = <FILE>;
# Go through each line of the GO_dev file. Split the line into array @current_line. Skip the line if it has evidence codes ISS, ISO, ISA, ISM, RCA, IGC, IEA, TAS, NAS, IC, ND, or NR.
##@lines = <FILE>;
##close(FILE);
while (<IN1>) {
	@lines = <IN1>;
	foreach my $line (@lines) {
		chomp $line;
		$line =~ tr/\n\r//d; #removes carriage returns
		my @current_line = split /\t/, $line;
		next if ($current_line[3] !~ /GO:/);

# If the current MGI ID is different from the last MGI ID, reset the array @goterms (contains all annotations for a given MGI/UniProt ID) for the new MGI/UniProt ID. This assures that the go terms for the previous MGI ID are not added to the go terms of the current MGI
		if ($last_UP ne $current_line[1]) {
			@annotations = ();
		}
# Check to make sure that every GO ID exists in the Ontology hash with an ontology designation of P, F, or C. If not, create the GO ID in the Ontology hash and give it an ontology of X for easy lookup later. @MissingPFC are GO IDs with no ontology in the file
		if (!exists $Ontology{$current_line[3]}) {
			$Ontology{$current_line[3]} = "X";
			push (@MissingPFC, $current_line[3])
		}
# If the UniProt ID is human, make a GAF2.0 annotation based on it
		if (exists $GeneHash{$current_line[1]}) {
			$current_UP = $current_line[1];
			$last_UP = $current_line[1];
			$two_IDs = $current_line[1];
			$current_annotation = join "\t", "UniProtKB", $current_line[1], $GeneHash{$current_UP}, $current_line[2], $current_line[3], "DFLAT GONE", $current_line[5], $current_line[1], $Ontology{$current_line[3]}, "", "", "", "taxon:9606", "20110126", "DFLAT", "original: $current_line[5]", $current_line[4];
			push (@annotations, $current_annotation);
			$AnnotationHash{$two_IDs} = join "\n", @annotations;
# If the UniProt isn't in %GeneHash already, it's not human. See if it's a mouse ortholog
		} else {
# If an annotation is going to be made from an ortholog, first things first. Check if it has an unacceptable evidence code
			next if (($current_line[5] =~ /IS/) or ($current_line[5] =~ /RCA/) or ($current_line[5] =~ /IGC/) or ($current_line[5] =~ /IEA/) or ($current_line[5] =~ /AS/)or ($current_line[5] =~ /IC/) or ($current_line[5] =~ /ND/) or ($current_line[5] =~ /NR/));
# If the Mouse UniProt ($current_line[2]) exists in the mouse UP/MGI hash %MUP from above, set $last_UP to the current UP
			if (exists $MUP{$current_line[1]}) {
				@MGIs = split /\t/, $MUP{$current_line[1]};
				foreach (@MGIs) {
					$current_MGI = $_;
					$last_UP = $current_line[1];
# @two_IDs and $two_IDs are the MGI ID and mouse UniProt Ortholog; this is a key for the %AnnotationHash
					if (exists $MGItoUP{$current_MGI}) {
						$current_UP = $MGItoUP{$current_MGI};
						$last_MGI = $current_MGI;
						my @two_IDs = ($current_MGI, $current_line[1]);
						$two_IDs = join "\n", @two_IDs;
# If the human UniProt ortholog ($current_UP) exists in %GeneHash, save the annotation information in modified GAF 2.0 format: UniProtKB, UniProt, Gene Symbol, blank or NOT, GO term ID, MGI Ortholog list, Evidence Code (ISS), original UniProt ID, Ontology (F, P, or C; X if missing), Gene description, blank, blank, Taxon, Date, DFLAT, Original Evidence Code, GO term
						if (exists $GeneHash{$current_UP}) {
							$current_annotation = join "\t", "UniProtKB", $MGItoUP{$current_MGI}, $GeneHash{$current_UP}, $current_line[2], $current_line[3], "DFLAT GONE with MGI Ortholog list", "ISS", $current_line[1], $Ontology{$current_line[3]}, "", "", "", "taxon:9606", "20110126", "DFLAT", "original: $current_line[5]", $current_line[4];
							push (@annotations, $current_annotation);
							$AnnotationHash{$two_IDs} = join "\n", @annotations;
# If the $current_UP isn't in %GeneHash, no gene symbol exists, so save the annotations to a separate %MissingAnnotationHash
						} else {
							$current_annotation = join "\t", "UniProtKB", $MGItoUP{$current_MGI}, "", $current_line[2], $current_line[3], "DFLAT GONE with MGI Ortholog list", "ISS", $current_line[1], $Ontology{$current_line[3]}, "", "", "", "taxon:9606", "20110126", "DFLAT", "original: $current_line[5]", $current_line[4];
							push (@annotations, $current_annotation);
							$MissingAnnotationHash{$two_IDs} = join "\n", @annotations;
#				push (@missingGS, $two_IDs);			#simple array of missing IDs if you want to print these later
						}
###			$AnnotationHash{$two_IDs} = join "\n", @annotations; 	#uncomment if you want all annotation together regardless of missing IDs
					}
				}
			}
		}
	}
}
# Write file of GO terms that don't have ontology (P, F, or C) 
open(FILE, ">Output/MissingPFC.GONE.txt");
foreach (@MissingPFC){
print FILE "$_\n";
}

# Write file of UniProt IDs that are missing Gene Symbols
open(FILE, ">Output/MissingGS.GONE.txt");
foreach $two_IDs (sort keys %MissingAnnotationHash){
	print FILE "$MissingAnnotationHash{$two_IDs}\n";
}

# Print the %annotation hash for the user
# Printed in GAF 2.0 format, except that the last 2 columns are the former ID and the go term text
##foreach $UPI (sort keys %Annotation) {
##	print "$Annotation{$UPI}\n\n";
##}
unless (open(FILE, ">Output/Orth_GAF2.0.GONE.txt")) {
	die "Couldn't open Orth_GAF2.0 to write!\n";
}
foreach $two_IDs (sort keys %AnnotationHash) {
	print FILE "$AnnotationHash{$two_IDs}\n";
}
close(FILE);
