#!/usr/bin/env perl -w
#! Perl -w
use strict;
use warnings;

# This program takes a file of mouse-human orthologs and a file of mouse development annotation, and converts the mouse annotation to the human ortholog. The program then looks up the ontology of each annotation using a terms/IDs file. Finally, the program uses a file containing a uniprot-to-official-gene-symbol file and inserts the official gene symbol into each annotation (and copies the annotation if there is more than one gene symbol for a given uniprot ID).
# The program produces 4 output files:
# MGI_UniProt_Orthologs.txt -- mouse MGI and their human UniProt orthologs
# MissingPFC.txt -- GO IDs that are missing from the terms/ID file that has their ontology
# MissingGS.txt -- UniProt IDs that don't have Gene Symbols (these annotations still appear in OrthGAF2.0.txt)
# OrthGAF2.0.txt -- The human developmental annotation, in GAF 2.0 format (except the last 2 columns which instead contain additional info: the original mouse evidence code and the text part of the GO term)


my $UniProt; #Human UniProt ID ortholog from Human_SP_Ids.txt (Value of hash %MGItoUP)
my $MGI; #Mouse MGI ID from Human_SP_Ids.txt (key of hash %MGItoUP)
my %MGItoUP; #Hash of mouse MGI keys and their human UniProt ID values


# Open the file that contains HGNC, UniProt, and MGI IDs in that order, tab-delimited.
# This file was obtained from MGI
my $genefile = "Input_Mappings/Human_SP_IDs.txt";
unless (open(FILE, $genefile)) {
    die "Couldn't open $genefile\n";
}

# Read in the header of $genefile
my $header = <FILE>;

# Each line is split by \t into 3 elements of an array, @IDs. Skip the line if the 2nd and 3rd element are undefined (ie there is no UniProt or MGI ID). Otherwise, the 3rd element is the MGI ID ($MGI) 2nd element is the UniProt ID ($UniProt) which become the keys and values, respectively, of the hash %HumanID.
my @lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
	my @current_line = split /\t/, $line;
      	next if (!($current_line[1]) or !($current_line[2]));
	$MGI = $current_line[2];
	$UniProt = $current_line[1];
	$MGItoUP{$MGI} = $UniProt;
	}
close(FILE);

# Print the hash of MGI/UniProt IDs for the user:
##foreach $MGI (sort keys %MGItoUP) {
##	print "$MGI\t$MGItoUP{$MGI}\n";
##}

# Write the hash of MGI and UniProt IDs to a file
unless (open(FILE, ">Output/MGI_UniProt_Orthologs.MGI.txt")) {
    die "Couldn't open MGI_UniProt_Orthologs to write!\n"
}
foreach $MGI (sort keys %MGItoUP) {
    print FILE "$MGI\t$MGItoUP{$MGI}\n";
}
close(FILE);



my $GO; #GO ID number; key for %ontology hash
my $PFC; #Ontology (P, F, or C); value for %ontology hash
my %Ontology; #Hash of GO IDs and their Ontologies
my @MissingPFC; #List of GO IDs that don't exist in the GO terms/ontology file

# Open the file that contains the P, F, or C ontology for each GO term
# This file was downloaded from the GO WHEN!? and updated manually 3.9.11
my $pfcfile = "Input_Mappings/GO.terms_alt_ids.txt"; #GO terms/ontology file
unless (open(FILE, $pfcfile)) {
	die "Couldn't open $pfcfile!\n";
}

# Read each line. Skip the header lines (start with !). Split each line and make a hash of the GO IDs as keys and their P, F, or C ontology as the values
@lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
      	next if ($line =~ /^!/);
	my @current_line = split /\t/, $line;
	$GO = $current_line[0];
	$PFC = $current_line[3];
	$Ontology{$GO} = $PFC;
	}
close(FILE);

# Print the Ontology hash for the user
##foreach $GO (keys %Ontology) {
##print "$GO\t$Ontology{$GO}\n";
##}



my %GeneHash; #A hash with UniProt IDs as keys and an array of gene symbols as keys
my $UP; #Human UniProt keys in the %GeneHash
my $GS; #Human GS values in the %GeneHash
my $current_UP; #current UniProt ID
my $current_annotation; #Annotation line being created in GAF2.0 format, derived from annotation file, for a given MGI/UniProt ortholog
my @annotations = (); #array of annotations ($current_annotation)s for each MGI/UniProt
my %AnnotationHash; #Hash of MGI/UniProt keys ($two_IDs) and annotation values (@annotations)
my $two_IDs; #MGI/UniProt key of %AnnotationHash
my $last_MGI = ""; #The previous MGI term; compared to current MGI ($currentPline[0] to reset @annotations if working with new MGI
my %MissingAnnotationHash; #Hash of annotations with missing GS (UniProt does not exist in %GeneHash
my @missingGS; #Array of UniProt IDs that don't have Gene Symbols

# Open the file that has the uniprot-genesymbol conversion (two columns, tab-delimited)
# This file has to be made by the user by submitting UniProt IDs to DAVID
my $GSfile = "Input_Mappings/uniprot-9606.tab";
unless (open(FILE, $GSfile)) {
	die "Couldn't open $GSfile\n";
}

# Split each line and create a hash with the UniProt ($UP) as the keys and the gene symbols ($GS) as the values. If the UniProt already exists in the hash, add the additional gene symbol to the value separated by a tab
$header = <FILE>;
@lines = <FILE>;
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
	($UP, $GS) = split /\s/, $line;
	next if (!$GS);
	$GeneHash{$UP} = $GS;
}
close(FILE);

# Print %GeneHash for the user
##foreach $UP (sort keys %GeneHash) {
##	print "$UP\t$GeneHash{$UP}\n";
##}

# Print %GeneHash to a file
unless (open(FILE, ">Output/UPtoGS.MGI.txt")) {
	die "Couldn't open Orth_GAF2.0 to write!\n";
}
foreach $UP (sort keys %GeneHash) {
	print FILE "$UP\t$GeneHash{$UP}\n";
}
close(FILE);

# Open the file that has the annotations you want to search through
# This file was obtained from MGI
# Please note that the GO_dev file has to be sorted by MGI ID (all annotations for a given ID must be blocked together)
my $annotationfile = "Input_Annotations/GO_developmentGenes.txt";
unless (open(FILE, $annotationfile)) {
	die "Couldn't open $annotationfile\n";
}

# Go through each line of the GO_dev file. Split the line into array @current_line. Skip the line if it has evidence codes ISS, ISO, ISA, ISM, IGC, **NEW** IBA, IBD, IKR, IRD **ENDNEW**, RCA, IEA, TAS, NAS, IC, ND, or NR.
@lines = <FILE>;
close(FILE);
foreach my $line (@lines) {
	chomp $line;
	$line =~ tr/\n\r//d; #removes carriage returns
	my @current_line = split /\t/, $line;
	next if (($current_line[8] =~ /IS/) or ($current_line[8] =~ /RCA/) or ($current_line[8] =~ /IGC/) or ($current_line[8] =~ /IEA/) or ($current_line[8] =~ /AS/)or ($current_line[8] =~ /IC/) or ($current_line[8] =~ /ND/) or ($current_line[8] =~ /NR/) or ($current_line[8] =~ /IBA/) or ($current_line[8] =~ /IBD/) or ($current_line[8] =~ /IKR/) or ($current_line[8] =~ /IRD/));
# If the current MGI ID is different from the last MGI ID, reset the array @goterms (contains all annotations for a given MGI/UniProt ID) for the new MGI/UniProt ID. This assures that the go terms for the previous MGI ID are not added to the go terms of the current MGI
	if ($last_MGI ne $current_line[2]) {
		@annotations = ();
	}
# Check to make sure that every GO ID exists in the Ontology hash with an ontology designation of P, F, or C. If not, create the GO ID in the Ontology hash and give it an ontology of X for easy lookup later. @MissingPFC are GO IDs with no ontology in the file
	if (!exists $Ontology{$current_line[6]}) {
		$Ontology{$current_line[6]} = "X";
		push (@MissingPFC, $current_line[6])
	}
# If the MGI ID ($current_line[2]) exists in the hash %MGItoUP from above, set $last_MGI to the current MGI ID
	if (exists $MGItoUP{$current_line[2]}) {
		$current_UP = $MGItoUP{$current_line[2]};
		$last_MGI = $current_line[2];
# @two_IDs and $two_IDs are the MGI ID and human UniProt Ortholog; this is a key for the %AnnotationHash
		my @two_IDs = ($current_line[2], $MGItoUP{$current_line[2]});
		$two_IDs = join "\n", @two_IDs;
# If the human UniProt ortholog ($current_UP) exists in %GeneHash, save the annotation information in modified GAF 2.0 format: UniProtKB, UniProt, Gene Symbol, GO term ID, MGI Ortholog list, Evidence Code (ISS), original MGI ID, Ontology (F, P, or C; X if missing), Gene description, Taxon, Date, DFLAT, Original Evidence Code, GO term
		if (exists $GeneHash{$current_UP}) {
			$current_annotation = join "\t", "UniProtKB", $MGItoUP{$current_line[2]}, $GeneHash{$current_UP}, "", $current_line[6], "MGI Ortholog list", "ISS", $current_line[2], $Ontology{$current_line[6]}, $current_line[4], "", "", "taxon:9606", "20110126", "DFLAT", "original: $current_line[8]", $current_line[7];
			push (@annotations, $current_annotation);
			$AnnotationHash{$two_IDs} = join "\n", @annotations;
# If the $current_UP isn't in %GeneHash, no gene symbol exists, so save the annotations to a separate %MissingAnnotationHash
		} else {
			$current_annotation = join "\t", "UniProtKB", $MGItoUP{$current_line[2]},"", "", $current_line[6], "MGI Ortholog list", "ISS", $current_line[2], $Ontology{$current_line[6]}, $current_line[4], "", "", "taxon:9606", "20110126", "DFLAT", "original: $current_line[8]", $current_line[7];
			push (@annotations, $current_annotation);
			$MissingAnnotationHash{$two_IDs} = join "\n", @annotations;
#			push (@missingGS, $two_IDs);			#simple array of missing IDs if you want to print these later
		}
###		$AnnotationHash{$two_IDs} = join "\n", @annotations; 	#uncomment if you want all annotation together regardless of missing IDs
	}
}

# Write file of GO terms that don't have ontology (P, F, or C) 
open(FILE, ">Output/MissingPFC.MGI.txt");
foreach (@MissingPFC){
print FILE "$_\n";
}

# Write file of UniProt IDs that are missing Gene Symbols
open(FILE, ">Output/MissingGS.MGI.txt");
foreach $two_IDs (sort keys %MissingAnnotationHash){
	print FILE "$MissingAnnotationHash{$two_IDs}\n";
}

# Print the %annotation hash for the user
# Printed in GAF 2.0 format, except that the last 2 columns are the former ID and the go term text
##foreach $UPI (sort keys %Annotation) {
##	print "$Annotation{$UPI}\n\n";
##}
unless (open(FILE, ">Output/Orth_GAF2.0.MGI.txt")) {
	die "Couldn't open Orth_GAF2.0 to write!\n";
}
foreach $two_IDs (sort keys %AnnotationHash) {
	print FILE "$AnnotationHash{$two_IDs}\n";
}
close(FILE);
