#!/bin/bash
# this file processes the pipeline to generate updated version of DFLAT.
# please make sure all input files are up-to-date and in place before running this script.

# 0) make sure the "Output" directory exists - if not, create one
if [ ! -d "Output" ]; then
	echo "--> 0) Output directory does not exists, creating one"
	mkdir Output
fi

# 1) delete DFLAT from gene_association.goa_human file
# input: gene_association.goa_human
# output: gene_association_noDFLAT.goa_human, DFLATonly.goa_human
echo "--> 1) Deleting DFLAT annotation from gene_association.goa_human"
perl DeleteDFLAT.pl

# 2) create GO.terms_alt_ids.txt file from .obo file
# input: go.obo 
# output: GO.terms_alt_ids.txt
echo "--> 2) Creating an input file that lists all GO terms and related information (GO.terms_alt_ids.txt)"
python makeGOalts.py Input_Annotations/go.obo Input_Mappings/GO.terms_alt_ids.txt

# 3) create mapping between MGI mouse annotation and orthologous human genes
# input: Human_SP_IDs.txt, uniprot-9606.tab, GO.terms_alt_ids.txt, GO_developmentGenes.txt
# output: MGI_Uniprot_Orthologs.MGI.txt, MissingPFC.MGI.txt, MissingGS.MGI.txt, OrthGAF2.0.MGI.txt, UPtoGS.MGI.txt
echo "--> 3) Creating mapping between MGI mouse annotation and orthologous human genes: running GAF2.0.MGI.pl"
perl GAF2.0.MGI.pl 

# 4) create mapping between any published DFLAT mouse annotation and orthologous human genes
# input: Human_SP_IDs.txt, uniprot-9606.tab, GO.terms_alt_ids.txt, P2GO4.10.13.txt, MGIMouseUP.txt
# output: MGI_Uniprot_Orthologs.DFLAT.txt, MissingPFC.DFLAT.txt, MissingGS.DFLAT.txt, OrthGAF2.0.DFLAT.txt, UPtoGS.DFLAT.txt, mouse_MGI_UP.DFLAT.txt
echo "--> 4) Creating mapping between published DFLAT mouse annotation and orthologous human genes: running GAF2.0.DFLAT.pl"
perl GAF2.0.DFLAT.pl Input_Annotations/P2GO4.10.13.txt

# 5) create mapping between any unpublished DFLAT annotation and orthologous human genes
# input: Human_SP_IDs.txt, uniprot-9606.tab, GO.terms_alt_ids.txt, GONE4.10.13.txt, MGIMouseUP.txt
# output: MGI_Uniprot_Orthologs.GONE.txt, MissingPFC.GONE.txt, MissingGS.GONE.txt, OrthGAF2.0.GONE.txt, UPtoGS.GONE.txt, mouse_MGI_UP.GONE.txt
echo "--> 5) Creating mapping between unpublished DFLAT mouse annotation and orthologous human genes: running GAF2.0.GONE.pl"
perl GAF2.0.GONE.pl Input_Annotations/GONE4.10.13.txt

# 6) search for missing gene symbols in MissingGS.tag.txt files and add them to annotation files
# input: MissingGS.tag.txt
# output: updated OrthGAF2.0.tag.txt
echo "--> 6) Adding missing gene symbols to annotation files created in step 3), 4), and 5)"
python addMissingGS.py

# 7) search for missing ontology or annotation in MissingPFC.tag.txt files and add them to annotation files
# input: MissingPFC.tag.txt
# output: updated OrthGAF2.0.tag.txt
# if GO.terms_alt_ids.txt file is comprehensive, this program should not be needed. 
echo "--> 7) Adding missing namespace information for some GO terms to annotation files created in step 3), 4), and 5)"
python addMissingPFC.py

# 8) combine all annotation files and generate one annotation file: 
# all from Orth_GAF2.0.MGI.txt, 
# lines with "DFLAT with MGI Ortholog list" from Orth_GAF2.0.DFLAT.txt, 
# and lines with "DFLAT GONE with MGI Ortholog list" from Orth_GAF2.0.GONE.txt
# input: Orth_GAF2.0.MGI.txt, Orth_GAF2.0.DFLAT.txt, Orth_GAF2.0.GONE.txt
# output: OrthOnly_GAF2.0.DFLAT.GONE.MGI.txt
echo "--> 8) Combining all annotation files created in 3), 4) and 5): generate OrthOnly_GAF2.0.DFLAT.GONE.MGI.txt"
cp Output/Orth_GAF2.0.MGI.txt Output/OrthOnly_GAF2.0.DFLAT.GONE.MGI.txt
awk -F "\t" '(NF>0&&$6=="DFLAT with MGI Ortholog list") {print}' Output/Orth_GAF2.0.DFLAT.txt >> Output/OrthOnly_GAF2.0.DFLAT.GONE.MGI.txt
awk -F "\t" '(NF>0&&$6=="DFLAT GONE with MGI Ortholog list") {print}' Output/Orth_GAF2.0.GONE.txt >> Output/OrthOnly_GAF2.0.DFLAT.GONE.MGI.txt

# 9) create a file consisting only the annotations to the developmental branch, filter annotation by user-inputed GO term
# input: go.obo, OrthOnly_GAF2.0.DFLAT.GONE.MGI.txt (raw orthologous annotation file)
# output: filtered.txt (contains only orthologous annotations that is in the developmental branch, GO:0032502)
echo "--> 9) Filtering annotations from combined annotation file that belong to developmental branch in GO hierarchy: generate OrthOnly_GAF2.0.DFLAT.GONE.MGI.txt"
g++ filterRAW.cpp
./a.out Input_Annotations/go.obo Output/OrthOnly_GAF2.0.DFLAT.GONE.MGI.txt

# 10) combine human DFLAT and GONE annotation, filtered orthologous annotation (in filtered.txt) with gene_association.goa_human
# lines without "DFLAT with MGI Ortholog list" from Orth_GAF2.0.DFLAT.txt,
# lines without "DFLAT GONE with MGI Ortholog list" from Orth_GAF2.0.GONE.txt,
# all lines in filtered.txt, and
# all lines in gene_association.goa_human
# input: Orth_GAF2.0.DFLAT.txt, Orth_GAF2.0.GONE.txt, filtered.txt, gene_association.goa_human
# output: gene_association_DFLAT.GONE.filteredOrth.goa_human
echo "--> 10) Combining annotations from DFLAT, GONE, and developmental branch"
cat Input_Annotations/gene_association.goa_human Output/filtered.txt > Output/gene_association_DFLAT.GONE.filteredOrth.goa_human
awk -F "\t" '(NF>0&&$6!="DFLAT with MGI Ortholog list") {print}' Output/Orth_GAF2.0.DFLAT.txt >> Output/gene_association_DFLAT.GONE.filteredOrth.goa_human
awk -F "\t" '(NF>0&&$6!="DFLAT GONE with MGI Ortholog list") {print}' Output/Orth_GAF2.0.GONE.txt >> Output/gene_association_DFLAT.GONE.filteredOrth.goa_human

# 11) create .gmt files from raw annotation, mapping annotation up in the GO hierarchy via is_a and has_part relationships
# input: go.obo, DFLATonly.goa_human
echo "--> 11) Creating .gmt files with transferring collected annotations to ancestors - no DFLAT, only goa"
echo "-->     output: f.GOA.gmt, p.GOA.gmt, c.GOA.gmt, fpc.GOA.gmt"
g++ humanGMT.cpp 
./a.out Output/gene_association_noDFLAT.goa_human Output/f.GOA.gmt Output/p.GOA.gmt Output/c.GOA.gmt Input_Annotations/go.obo Output/fpc.GOA.gmt
echo ""
echo "--> 11) Creating .gmt files with transferring collected annotations to ancestors - DFLAT"
echo "-->     output: f.DFLAT.gmt, p.DFLAT.gmt, c.DFLAT.gmt, fpc.DFLAT.gmt"
./a.out Output/gene_association_DFLAT.GONE.filteredOrth.goa_human Output/f.DFLAT.gmt Output/p.DFLAT.gmt Output/c.DFLAT.gmt Input_Annotations/go.obo Output/fpc.DFLAT.gmt
echo

# 12) clean up directory
echo "--> 12) Cleaning up the working directory..."
rm dummy*.txt
rm clean.txt
